@@ -1,29 +1,17 @@
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#    http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-# implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# This file is dual licensed under the terms of the Apache License, Version
+# 2.0, and the BSD License. See the LICENSE file in the root of this repository
+# for complete details.
 
 from __future__ import absolute_import, division, print_function
 
-import pytest
-
 from cryptography import utils
 from cryptography.exceptions import (
     UnsupportedAlgorithm, _Reasons
 )
 from cryptography.hazmat.backends.interfaces import (
-    CMACBackend, CipherBackend, DSABackend, EllipticCurveBackend, HMACBackend,
-    HashBackend, PBKDF2HMACBackend, PEMSerializationBackend,
-    PKCS8SerializationBackend, RSABackend,
-    TraditionalOpenSSLSerializationBackend
+    CMACBackend, CipherBackend, DERSerializationBackend, DSABackend,
+    EllipticCurveBackend, HMACBackend, HashBackend, PBKDF2HMACBackend,
+    PEMSerializationBackend, RSABackend, X509Backend
 )
 from cryptography.hazmat.backends.multibackend import MultiBackend
 from cryptography.hazmat.primitives import cmac, hashes, hmac
@@ -38,15 +26,15 @@ class DummyCipherBackend(object):
     def __init__(self, supported_ciphers):
         self._ciphers = supported_ciphers
 
-    def cipher_supported(self, algorithm, mode):
-        return (type(algorithm), type(mode)) in self._ciphers
+    def cipher_supported(self, cipher, mode):
+        return (type(cipher), type(mode)) in self._ciphers
 
-    def create_symmetric_encryption_ctx(self, algorithm, mode):
-        if not self.cipher_supported(algorithm, mode):
+    def create_symmetric_encryption_ctx(self, cipher, mode):
+        if not self.cipher_supported(cipher, mode):
             raise UnsupportedAlgorithm("", _Reasons.UNSUPPORTED_CIPHER)
 
-    def create_symmetric_decryption_ctx(self, algorithm, mode):
-        if not self.cipher_supported(algorithm, mode):
+    def create_symmetric_decryption_ctx(self, cipher, mode):
+        if not self.cipher_supported(cipher, mode):
             raise UnsupportedAlgorithm("", _Reasons.UNSUPPORTED_CIPHER)
 
 
@@ -92,17 +80,7 @@ class DummyPBKDF2HMACBackend(object):
 
 @utils.register_interface(RSABackend)
 class DummyRSABackend(object):
-    def generate_rsa_private_key(self, public_exponent, private_key):
-        pass
-
-    def create_rsa_signature_ctx(self, private_key, padding, algorithm):
-        pass
-
-    def create_rsa_verification_ctx(self, public_key, signature, padding,
-                                    algorithm):
-        pass
-
-    def mgf1_hash_supported(self, algorithm):
+    def generate_rsa_private_key(self, public_exponent, key_size):
         pass
 
     def rsa_padding_supported(self, padding):
@@ -111,12 +89,6 @@ class DummyRSABackend(object):
     def generate_rsa_parameters_supported(self, public_exponent, key_size):
         pass
 
-    def decrypt_rsa(self, private_key, ciphertext, padding):
-        pass
-
-    def encrypt_rsa(self, public_key, plaintext, padding):
-        pass
-
     def load_rsa_private_numbers(self, numbers):
         pass
 
@@ -135,12 +107,6 @@ class DummyDSABackend(object):
     def generate_dsa_private_key_and_parameters(self, key_size):
         pass
 
-    def create_dsa_signature_ctx(self, private_key, algorithm):
-        pass
-
-    def create_dsa_verification_ctx(self, public_key, signature, algorithm):
-        pass
-
     def dsa_hash_supported(self, algorithm):
         pass
 
@@ -153,6 +119,9 @@ class DummyDSABackend(object):
     def load_dsa_public_numbers(self, numbers):
         pass
 
+    def load_dsa_parameter_numbers(self, numbers):
+        pass
+
 
 @utils.register_interface(CMACBackend)
 class DummyCMACBackend(object):
@@ -197,31 +166,11 @@ class DummyEllipticCurveBackend(object):
         if not self.elliptic_curve_supported(numbers.public_numbers.curve):
             raise UnsupportedAlgorithm(_Reasons.UNSUPPORTED_ELLIPTIC_CURVE)
 
-    def elliptic_curve_private_key_from_numbers(self, numbers):
-        if not self.elliptic_curve_supported(numbers.public_numbers.curve):
-            raise UnsupportedAlgorithm(_Reasons.UNSUPPORTED_ELLIPTIC_CURVE)
-
-    def elliptic_curve_public_key_from_numbers(self, numbers):
-        if not self.elliptic_curve_supported(numbers.curve):
-            raise UnsupportedAlgorithm(_Reasons.UNSUPPORTED_ELLIPTIC_CURVE)
-
     def load_elliptic_curve_public_numbers(self, numbers):
         if not self.elliptic_curve_supported(numbers.curve):
             raise UnsupportedAlgorithm(_Reasons.UNSUPPORTED_ELLIPTIC_CURVE)
 
 
-@utils.register_interface(PKCS8SerializationBackend)
-class DummyPKCS8SerializationBackend(object):
-    def load_pkcs8_pem_private_key(self, data, password):
-        pass
-
-
-@utils.register_interface(TraditionalOpenSSLSerializationBackend)
-class DummyTraditionalOpenSSLSerializationBackend(object):
-    def load_traditional_openssl_pem_private_key(self, data, password):
-        pass
-
-
 @utils.register_interface(PEMSerializationBackend)
 class DummyPEMSerializationBackend(object):
     def load_pem_private_key(self, data, password):
@@ -231,6 +180,24 @@ class DummyPEMSerializationBackend(object):
         pass
 
 
+@utils.register_interface(DERSerializationBackend)
+class DummyDERSerializationBackend(object):
+    def load_der_private_key(self, data, password):
+        pass
+
+    def load_der_public_key(self, data):
+        pass
+
+
+@utils.register_interface(X509Backend)
+class DummyX509Backend(object):
+    def load_pem_x509_certificate(self, data):
+        pass
+
+    def load_der_x509_certificate(self, data):
+        pass
+
+
 class TestMultiBackend(object):
     def test_ciphers(self):
         backend = MultiBackend([
@@ -303,22 +270,10 @@ class TestMultiBackend(object):
             key_size=1024, public_exponent=65537
         )
 
-        backend.create_rsa_signature_ctx("private_key", padding.PKCS1v15(),
-                                         hashes.MD5())
-
-        backend.create_rsa_verification_ctx("public_key", "sig",
-                                            padding.PKCS1v15(), hashes.MD5())
-
-        backend.mgf1_hash_supported(hashes.MD5())
-
         backend.rsa_padding_supported(padding.PKCS1v15())
 
         backend.generate_rsa_parameters_supported(65537, 1024)
 
-        backend.encrypt_rsa("public_key", "encryptme", padding.PKCS1v15())
-
-        backend.decrypt_rsa("private_key", "encrypted", padding.PKCS1v15())
-
         backend.load_rsa_private_numbers("private_numbers")
 
         backend.load_rsa_public_numbers("public_numbers")
@@ -329,23 +284,6 @@ class TestMultiBackend(object):
         ):
             backend.generate_rsa_private_key(key_size=1024, public_exponent=3)
 
-        with raises_unsupported_algorithm(
-            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
-            backend.create_rsa_signature_ctx("private_key", padding.PKCS1v15(),
-                                             hashes.MD5())
-
-        with raises_unsupported_algorithm(
-            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
-            backend.create_rsa_verification_ctx(
-                "public_key", "sig", padding.PKCS1v15(), hashes.MD5())
-
-        with raises_unsupported_algorithm(
-            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
-            backend.mgf1_hash_supported(hashes.MD5())
-
         with raises_unsupported_algorithm(
             _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
         ):
@@ -356,16 +294,6 @@ class TestMultiBackend(object):
         ):
             backend.generate_rsa_parameters_supported(65537, 1024)
 
-        with raises_unsupported_algorithm(
-            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
-            backend.encrypt_rsa("public_key", "encryptme", padding.PKCS1v15())
-
-        with raises_unsupported_algorithm(
-            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
-            backend.decrypt_rsa("private_key", "encrypted", padding.PKCS1v15())
-
         with raises_unsupported_algorithm(
             _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
         ):
@@ -387,12 +315,11 @@ class TestMultiBackend(object):
         backend.generate_dsa_private_key(parameters)
         backend.generate_dsa_private_key_and_parameters(key_size=1024)
 
-        backend.create_dsa_verification_ctx("public_key", "sig", hashes.SHA1())
-        backend.create_dsa_signature_ctx("private_key", hashes.SHA1())
         backend.dsa_hash_supported(hashes.SHA1())
         backend.dsa_parameters_supported(1, 2, 3)
         backend.load_dsa_private_numbers("numbers")
         backend.load_dsa_public_numbers("numbers")
+        backend.load_dsa_parameter_numbers("numbers")
 
         backend = MultiBackend([])
         with raises_unsupported_algorithm(
@@ -410,18 +337,6 @@ class TestMultiBackend(object):
         ):
             backend.generate_dsa_private_key_and_parameters(key_size=1024)
 
-        with raises_unsupported_algorithm(
-            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
-            backend.create_dsa_signature_ctx("private_key", hashes.SHA1())
-
-        with raises_unsupported_algorithm(
-            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
-            backend.create_dsa_verification_ctx(
-                "public_key", b"sig", hashes.SHA1()
-            )
-
         with raises_unsupported_algorithm(
             _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
         ):
@@ -442,6 +357,11 @@ class TestMultiBackend(object):
         ):
             backend.load_dsa_public_numbers("numbers")
 
+        with raises_unsupported_algorithm(
+            _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
+        ):
+            backend.load_dsa_parameter_numbers("numbers")
+
     def test_cmac(self):
         backend = MultiBackend([
             DummyCMACBackend([algorithms.AES])
@@ -523,69 +443,6 @@ class TestMultiBackend(object):
                 )
             )
 
-    def test_deprecated_elliptic_curve(self):
-        backend = MultiBackend([
-            DummyEllipticCurveBackend([
-                ec.SECT283K1
-            ])
-        ])
-
-        assert backend.elliptic_curve_signature_algorithm_supported(
-            ec.ECDSA(hashes.SHA256()),
-            ec.SECT163K1()
-        ) is False
-
-        pub_numbers = ec.EllipticCurvePublicNumbers(2, 3, ec.SECT283K1())
-        numbers = ec.EllipticCurvePrivateNumbers(1, pub_numbers)
-
-        pytest.deprecated_call(
-            backend.elliptic_curve_private_key_from_numbers,
-            numbers
-        )
-        pytest.deprecated_call(
-            backend.elliptic_curve_public_key_from_numbers,
-            pub_numbers
-        )
-
-        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_ELLIPTIC_CURVE):
-            backend.elliptic_curve_private_key_from_numbers(
-                ec.EllipticCurvePrivateNumbers(
-                    1,
-                    ec.EllipticCurvePublicNumbers(
-                        2,
-                        3,
-                        ec.SECT163K1()
-                    )
-                )
-            )
-
-        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_ELLIPTIC_CURVE):
-            backend.elliptic_curve_public_key_from_numbers(
-                ec.EllipticCurvePublicNumbers(
-                    2,
-                    3,
-                    ec.SECT163K1()
-                )
-            )
-
-    def test_pkcs8_serialization_backend(self):
-        backend = MultiBackend([DummyPKCS8SerializationBackend()])
-
-        backend.load_pkcs8_pem_private_key(b"keydata", None)
-
-        backend = MultiBackend([])
-        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
-            backend.load_pkcs8_pem_private_key(b"keydata", None)
-
-    def test_traditional_openssl_serialization_backend(self):
-        backend = MultiBackend([DummyTraditionalOpenSSLSerializationBackend()])
-
-        backend.load_traditional_openssl_pem_private_key(b"keydata", None)
-
-        backend = MultiBackend([])
-        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
-            backend.load_traditional_openssl_pem_private_key(b"keydata", None)
-
     def test_pem_serialization_backend(self):
         backend = MultiBackend([DummyPEMSerializationBackend()])
 
@@ -597,3 +454,27 @@ class TestMultiBackend(object):
             backend.load_pem_private_key(b"keydata", None)
         with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
             backend.load_pem_public_key(b"keydata")
+
+    def test_der_serialization_backend(self):
+        backend = MultiBackend([DummyDERSerializationBackend()])
+
+        backend.load_der_private_key(b"keydata", None)
+        backend.load_der_public_key(b"keydata")
+
+        backend = MultiBackend([])
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
+            backend.load_der_private_key(b"keydata", None)
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
+            backend.load_der_public_key(b"keydata")
+
+    def test_x509_backend(self):
+        backend = MultiBackend([DummyX509Backend()])
+
+        backend.load_pem_x509_certificate(b"certdata")
+        backend.load_der_x509_certificate(b"certdata")
+
+        backend = MultiBackend([])
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_X509):
+            backend.load_pem_x509_certificate(b"certdata")
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_X509):
+            backend.load_der_x509_certificate(b"certdata")
